@@ -21,9 +21,6 @@
  *****************************************************************************/
 // $Id$
 
-// define before any includes
-#define BOOST_SPIRIT_THREADSAFE
-
 #include <mapnik/map.hpp>
 #include <mapnik/datasource_cache.hpp>
 #include <mapnik/font_engine_freetype.hpp>
@@ -33,6 +30,12 @@
 #include <mapnik/image_util.hpp>
 #include <mapnik/config_error.hpp>
 
+#if defined(HAVE_CAIRO)
+// cairo
+#include <mapnik/cairo_renderer.hpp>
+#include <cairomm/surface.h>
+#endif
+
 #include <iostream>
 
 
@@ -40,7 +43,8 @@ int main ( int argc , char** argv)
 {    
     if (argc != 2)
     {
-        std::cout << "usage: ./rundemo <mapnik_install_dir>\n";
+        std::cout << "usage: ./rundemo <mapnik_install_dir>\nUsually /usr/local/lib/mapnik\n";
+        std::cout << "Warning: ./rundemo looks for data in ../data/,\nTherefore must be run from within the demo/c++ folder.\n";
         return EXIT_SUCCESS;
     }
     
@@ -48,8 +52,10 @@ int main ( int argc , char** argv)
     try {
         std::cout << " running demo ... \n";
         std::string mapnik_dir(argv[1]);
-        datasource_cache::instance()->register_datasources(mapnik_dir + "/lib/mapnik/input/"); 
-        freetype_engine::register_font(mapnik_dir + "/lib/mapnik/fonts/DejaVuSans.ttf");
+        std::cout << " looking for 'shape.input' plugin in... " << mapnik_dir << "/input/" << "\n";
+        datasource_cache::instance()->register_datasources(mapnik_dir + "/input/"); 
+        std::cout << " looking for DejaVuSans font in... " << mapnik_dir << "/fonts/DejaVuSans.ttf" << "\n";
+        freetype_engine::register_font(mapnik_dir + "/fonts/DejaVuSans.ttf");
         
         Map m(800,600);
         m.set_background(color_factory::from_string("white"));
@@ -61,12 +67,12 @@ int main ( int argc , char** argv)
        
         rule_type provpoly_rule_on;
         provpoly_rule_on.set_filter(create_filter("[NAME_EN] = 'Ontario'"));
-        provpoly_rule_on.append(polygon_symbolizer(Color(250, 190, 183)));
+        provpoly_rule_on.append(polygon_symbolizer(color(250, 190, 183)));
         provpoly_style.add_rule(provpoly_rule_on);
         
         rule_type provpoly_rule_qc;
-        provpoly_rule_qc.set_filter(create_filter("[NAME_EN] = 'Quebec'"));
-        provpoly_rule_qc.append(polygon_symbolizer(Color(217, 235, 203)));
+        provpoly_rule_qc.set_filter(create_filter("[NOM_FR] = 'Québec'"));
+        provpoly_rule_qc.append(polygon_symbolizer(color(217, 235, 203)));
         provpoly_style.add_rule(provpoly_rule_qc);
         
         m.insert_style("provinces",provpoly_style);
@@ -74,7 +80,7 @@ int main ( int argc , char** argv)
         // Provinces (polyline)
         feature_type_style provlines_style;
         
-        stroke provlines_stk (Color(0,0,0),1.0);
+        stroke provlines_stk (color(0,0,0),1.0);
         provlines_stk.add_dash(8, 4);
         provlines_stk.add_dash(2, 2);
         provlines_stk.add_dash(2, 2);
@@ -90,7 +96,7 @@ int main ( int argc , char** argv)
         
         rule_type qcdrain_rule;
         qcdrain_rule.set_filter(create_filter("[HYC] = 8"));
-        qcdrain_rule.append(polygon_symbolizer(Color(153, 204, 255)));
+        qcdrain_rule.append(polygon_symbolizer(color(153, 204, 255)));
         qcdrain_style.add_rule(qcdrain_rule);
         
         m.insert_style("drainage",qcdrain_style);
@@ -99,7 +105,7 @@ int main ( int argc , char** argv)
         feature_type_style roads34_style;    
         rule_type roads34_rule;
         roads34_rule.set_filter(create_filter("[CLASS] = 3 or [CLASS] = 4"));
-        stroke roads34_rule_stk(Color(171,158,137),2.0);
+        stroke roads34_rule_stk(color(171,158,137),2.0);
         roads34_rule_stk.set_line_cap(ROUND_CAP);
         roads34_rule_stk.set_line_join(ROUND_JOIN);
         roads34_rule.append(line_symbolizer(roads34_rule_stk));
@@ -112,7 +118,7 @@ int main ( int argc , char** argv)
         feature_type_style roads2_style_1;
         rule_type roads2_rule_1;
         roads2_rule_1.set_filter(create_filter("[CLASS] = 2"));
-        stroke roads2_rule_stk_1(Color(171,158,137),4.0);
+        stroke roads2_rule_stk_1(color(171,158,137),4.0);
         roads2_rule_stk_1.set_line_cap(ROUND_CAP);
         roads2_rule_stk_1.set_line_join(ROUND_JOIN);
         roads2_rule_1.append(line_symbolizer(roads2_rule_stk_1));
@@ -123,7 +129,7 @@ int main ( int argc , char** argv)
         feature_type_style roads2_style_2;
         rule_type roads2_rule_2;
         roads2_rule_2.set_filter(create_filter("[CLASS] = 2"));
-        stroke roads2_rule_stk_2(Color(255,250,115),2.0);
+        stroke roads2_rule_stk_2(color(255,250,115),2.0);
         roads2_rule_stk_2.set_line_cap(ROUND_CAP);
         roads2_rule_stk_2.set_line_join(ROUND_JOIN);
         roads2_rule_2.append(line_symbolizer(roads2_rule_stk_2));
@@ -135,7 +141,7 @@ int main ( int argc , char** argv)
         feature_type_style roads1_style_1;
         rule_type roads1_rule_1;
         roads1_rule_1.set_filter(create_filter("[CLASS] = 1"));
-        stroke roads1_rule_stk_1(Color(188,149,28),7.0);
+        stroke roads1_rule_stk_1(color(188,149,28),7.0);
         roads1_rule_stk_1.set_line_cap(ROUND_CAP);
         roads1_rule_stk_1.set_line_join(ROUND_JOIN);
         roads1_rule_1.append(line_symbolizer(roads1_rule_stk_1));
@@ -145,7 +151,7 @@ int main ( int argc , char** argv)
         feature_type_style roads1_style_2;
         rule_type roads1_rule_2;
         roads1_rule_2.set_filter(create_filter("[CLASS] = 1"));
-        stroke roads1_rule_stk_2(Color(242,191,36),5.0);
+        stroke roads1_rule_stk_2(color(242,191,36),5.0);
         roads1_rule_stk_2.set_line_cap(ROUND_CAP);
         roads1_rule_stk_2.set_line_join(ROUND_JOIN);
         roads1_rule_2.append(line_symbolizer(roads1_rule_stk_2));
@@ -156,8 +162,8 @@ int main ( int argc , char** argv)
         
         feature_type_style popplaces_style;
         rule_type popplaces_rule;
-        text_symbolizer popplaces_text_symbolizer("GEONAME","DejaVu Sans Book",10,Color(0,0,0));
-        popplaces_text_symbolizer.set_halo_fill(Color(255,255,200));
+        text_symbolizer popplaces_text_symbolizer("GEONAME","DejaVu Sans Book",10,color(0,0,0));
+        popplaces_text_symbolizer.set_halo_fill(color(255,255,200));
         popplaces_text_symbolizer.set_halo_radius(1);
         popplaces_rule.append(popplaces_text_symbolizer);
         popplaces_style.add_rule(popplaces_rule);
@@ -247,20 +253,49 @@ int main ( int argc , char** argv)
         save_to_file<ImageData32>(buf.data(),"demo.jpg","jpeg");
         save_to_file<ImageData32>(buf.data(),"demo.png","png");
         save_to_file<ImageData32>(buf.data(),"demo256.png","png256");
-        std::cout << "Three maps have been rendered in the current directory:\n"
+        std::cout << "Three maps have been rendered using AGG in the current directory:\n"
            "- demo.jpg\n"
            "- demo.png\n"
            "- demo256.png\n"
            "Have a look!\n";
+
+        #if defined(HAVE_CAIRO)
+        Cairo::RefPtr<Cairo::ImageSurface> image_surface;
+
+        image_surface = Cairo::ImageSurface::create(Cairo::FORMAT_ARGB32, m.getWidth(),m.getHeight());
+        cairo_renderer<Cairo::Surface> png_render(m, image_surface);
+        png_render.apply();
+        image_surface->write_to_png("cairo-demo.png");
+
+        Image32 im(image_surface);
+        save_to_file(im, "cairo-demo256.png","png256");
+
+        Cairo::RefPtr<Cairo::Surface> surface;
+        surface = Cairo::PdfSurface::create("cairo-demo.pdf", m.getWidth(),m.getHeight());
+        cairo_renderer<Cairo::Surface> pdf_render(m, surface);
+        pdf_render.apply();
+
+        surface = Cairo::SvgSurface::create("cairo-demo.svg", m.getWidth(),m.getHeight());
+        cairo_renderer<Cairo::Surface> svg_render(m, surface);
+        svg_render.apply();
+
+        std::cout << "Three maps have been rendered using Cairo in the current directory:\n"
+           "- cairo-demo.png\n"
+           "- cairo-demo256.png\n"
+           "- cairo-demo.pdf\n"
+           "- cairo-demo.svg\n"
+           "Have a look!\n";
+        #endif
+
     }
     catch ( const mapnik::config_error & ex )
     {
-        std::cerr << "### Configuration error: " << ex.what();
+        std::cerr << "### Configuration error: " << ex.what() << std::endl;
         return EXIT_FAILURE;
     }
     catch ( const std::exception & ex )
     {
-        std::cerr << "### std::exception: " << ex.what();
+        std::cerr << "### std::exception: " << ex.what() << std::endl;
         return EXIT_FAILURE;
     }
     catch ( ... )
