@@ -35,21 +35,120 @@
 #include <string>
 #include <cassert>
 
+// cairo
+#ifdef HAVE_CAIRO
+#include <cairomm/surface.h>
+#endif
+
 namespace mapnik
 {
+   
+   struct Multiply
+   {
+      inline static void mergeRGB(unsigned const &r0, unsigned const &g0, unsigned const &b0,
+                                  unsigned &r1, unsigned &g1, unsigned &b1)
+      {
+         r1 = r1*r0/255;
+         g1 = g1*g0/255;
+         b1 = b1*b0/255;
+      }
+   };
+   struct Multiply2
+   {
+      inline static void mergeRGB(unsigned const &r0, unsigned const &g0, unsigned const &b0,
+                                  unsigned &r1, unsigned &g1, unsigned &b1)
+      {
+         r1 = r1*r0/128;
+         if (r1>255) r1=255;
+         g1 = g1*g0/128;
+         if (g1>255) g1=255;
+         b1 = b1*b0/128;
+         if (b1>255) b1=255;
+                }
+   };
+   struct Divide
+   {
+      inline static void mergeRGB(unsigned const &r0, unsigned const &g0, unsigned const &b0,
+                                  unsigned &r1, unsigned &g1, unsigned &b1)
+      {
+         r1 = r0*256/(r1+1);
+         g1 = g0*256/(g1+1);
+         b1 = b0*256/(b1+1);
+      }
+   };
+   struct Divide2
+   {
+      inline static void mergeRGB(unsigned const &r0, unsigned const &g0, unsigned const &b0,
+                                  unsigned &r1, unsigned &g1, unsigned &b1)
+      {
+         r1 = r0*128/(r1+1);
+         g1 = g0*128/(g1+1);
+         b1 = b0*128/(b1+1);
+      }
+   };
+   struct Screen
+        {
+                inline static void mergeRGB(unsigned const &r0, unsigned const &g0, unsigned const &b0,
+                                            unsigned &r1, unsigned &g1, unsigned &b1)
+                {
+                      r1 = 255 - (255-r0)*(255-r1)/255;
+                      g1 = 255 - (255-g0)*(255-g1)/255;
+                      b1 = 255 - (255-b0)*(255-b1)/255;
+                }
+        };
+        struct HardLight
+        {
+                inline static void mergeRGB(unsigned const &r0, unsigned const &g0, unsigned const &b0,
+                                            unsigned &r1, unsigned &g1, unsigned &b1)
+                {
+                      r1 = (r1>128)?255-(255-r0)*(255-2*(r1-128))/256:r0*r1*2/256;
+                      g1 = (g1>128)?255-(255-g0)*(255-2*(g1-128))/256:g0*g1*2/256;
+                      b1 = (b1>128)?255-(255-b0)*(255-2*(b1-128))/256:b0*b1*2/256;
+                }
+        };
+        struct MergeGrain
+        {
+                inline static void mergeRGB(unsigned const &r0, unsigned const &g0, unsigned const &b0,
+                                            unsigned &r1, unsigned &g1, unsigned &b1)
+                {
+                      r1 = (r1+r0>128)?r1+r0-128:0;
+                      if (r1>255) r1=255;
+                      g1 = (g1+g0>128)?g1+g0-128:0;
+                      if (g1>255) g1=255;
+                      b1 = (b1+b0>128)?b1+b0-128:0;
+                      if (b1>255) b1=255;
+                }
+        };
+        struct MergeGrain2
+        {
+                inline static void mergeRGB(unsigned const &r0, unsigned const &g0, unsigned const &b0,
+                                            unsigned &r1, unsigned &g1, unsigned &b1)
+                {
+                      r1 = (2*r1+r0>256)?2*r1+r0-256:0;
+                      if (r1>255) r1=255;
+                      g1 = (2*g1+g0>256)?2*g1+g0-256:0;
+                      if (g1>255) g1=255;
+                      b1 = (2*b1+b0>256)?2*b1+b0-256:0;
+                      if (b1>255) b1=255;
+                }
+        };
+   
     class MAPNIK_DECL Image32
     {
     private:
         unsigned width_;
         unsigned height_;
-        Color background_;
+        color background_;
         ImageData32 data_;
     public:
         Image32(int width,int height);
         Image32(Image32 const& rhs);
+#ifdef HAVE_CAIRO
+        Image32(Cairo::RefPtr<Cairo::ImageSurface> rhs);
+#endif
         ~Image32();
-        void setBackground(Color const& background);
-        const Color& getBackground() const;     
+        void setBackground(color const& background);
+        const color& getBackground() const;     
         const ImageData32& data() const;
         
         inline ImageData32& data() 
@@ -226,6 +325,48 @@ namespace mapnik
                 }
             }
         }
+        
+        template <typename MergeMethod>
+        inline void merge_rectangle(ImageData32 const& data, unsigned x0, unsigned y0, float opacity)
+        {
+            Envelope<int> ext0(0,0,width_,height_);
+            Envelope<int> ext1(x0,y0,x0 + data.width(),y0 + data.height());
+
+            if (ext0.intersects(ext1))
+            {
+                Envelope<int> box = ext0.intersect(ext1);
+                for (int y = box.miny(); y < box.maxy(); ++y)
+                {
+                   unsigned int* row_to =  data_.getRow(y);
+                   unsigned int const * row_from = data.getRow(y-y0);
+                   for (int x = box.minx(); x < box.maxx(); ++x)
+                   {
+                      unsigned rgba0 = row_to[x];
+                      unsigned rgba1 = row_from[x-x0];
+                      unsigned a1 = int( ((rgba1 >> 24) & 0xff) * opacity );
+                      if (a1 == 0) continue;
+                      unsigned r1 = rgba1 & 0xff;
+                      unsigned g1 = (rgba1 >> 8 ) & 0xff;
+                      unsigned b1 = (rgba1 >> 16) & 0xff;
+
+                      unsigned a0 = (rgba0 >> 24) & 0xff;
+                      unsigned r0 = rgba0 & 0xff ;
+                      unsigned g0 = (rgba0 >> 8 ) & 0xff;
+                      unsigned b0 = (rgba0 >> 16) & 0xff;
+
+                      unsigned a = (a1 * 255 + (255 - a1) * a0 + 127)/255;
+
+                      MergeMethod::mergeRGB(r0,g0,b0,r1,g1,b1);
+
+                      r0 = (r1*a1 + (((255 - a1) * a0 + 127)/255) * r0 + 127)/a;
+                      g0 = (g1*a1 + (((255 - a1) * a0 + 127)/255) * g0 + 127)/a;
+                      b0 = (b1*a1 + (((255 - a1) * a0 + 127)/255) * b0 + 127)/a;
+
+                      row_to[x] = (a << 24)| (b0 << 16) |  (g0 << 8) | (r0) ;
+                    }
+                }
+            }
+        }
     };
 }
 #endif //GRAPHICS_HPP
