@@ -31,7 +31,7 @@
 
 // boost
 #include <boost/algorithm/string.hpp>
-
+#include <boost/lexical_cast.hpp>
 // stl
 #include <string>
 
@@ -62,6 +62,10 @@ namespace mapnik {
    MAPNIK_DECL void save_to_file(T const& image,
                                  std::string const& filename);
    
+   template <typename T>
+   MAPNIK_DECL std::string save_to_string(T const& image,
+                                 std::string const& type);
+
    template <typename T>
    void save_as_png(T const& image,
                     std::string const& filename);
@@ -95,13 +99,22 @@ namespace mapnik {
       if (is_tiff(filename)) return "tiff";
       return "unknown";
    }
-   
+
+   inline std::string guess_type( const std::string & filename )
+   {
+      std::string::size_type idx = filename.find_last_of(".");
+      if ( idx != std::string::npos ) {
+          return filename.substr( idx + 1 );
+      }
+      return "<unknown>";
+   }
+       
    template <typename T>
    double distance(T x0,T y0,T x1,T y1)
    {
       double dx = x1-x0;
       double dy = y1-y0;
-      return sqrt(dx * dx + dy * dy);
+      return std::sqrt(dx * dx + dy * dy);
    }
    
    template <typename Image>
@@ -268,6 +281,138 @@ namespace mapnik {
       }
    }
    
+   template <typename Image>
+   inline void scale_image_bilinear (Image& target,const Image& source)
+   {
+
+      int source_width=source.width();
+      int source_height=source.height();
+
+      int target_width=target.width();
+      int target_height=target.height();
+
+      if (source_width<1 || source_height<1 ||
+          target_width<1 || target_height<1) return;
+      int x=0,y=0,xs=0,ys=0;
+      int tw2 = target_width/2;
+      int th2 = target_height/2;
+
+
+      for (y=0;y<target_height;++y)
+      {
+        ys = y*source_height/target_height;
+        int ys1 = ys+1;
+        if (ys1>=source_height)
+            ys1--;
+        unsigned yprt = y*source_height%target_height;
+        unsigned yprt1 = target_height-yprt;
+        for (x=0;x<target_width;++x)
+        {
+            xs = x*source_width/target_width;
+            if (source_width>=target_width || source_height>=target_height){
+                target(x,y)=source(xs,ys);
+                continue;
+            }
+            unsigned xprt = x*source_width%target_width;
+            unsigned xprt1 = target_width-xprt;
+            int xs1 = xs+1;
+            if (xs1>=source_width)
+                xs1--;
+
+            unsigned a = source(xs,ys);
+            unsigned b = source(xs1,ys);
+            unsigned c = source(xs,ys1);
+            unsigned d = source(xs1,ys1);
+            unsigned out=0;
+            unsigned t = 0;
+
+            for(int i=0; i<4; i++){
+                unsigned p,r,s;
+                // X axis
+                p = a&0xff;
+                r = b&0xff;
+                if (p!=r)
+                    r = (r*xprt+p*xprt1+tw2)/target_width;
+                p = c&0xff;
+                s = d&0xff;
+                if (p!=s)
+                    s = (s*xprt+p*xprt1+tw2)/target_width;
+                // Y axis
+                if (r!=s)
+                    r = (s*yprt+r*yprt1+th2)/target_height;
+                // channel up
+                out |= r << t;
+                t += 8;
+                a >>= 8;
+                b >>= 8;
+                c >>= 8;
+                d >>= 8;
+            }
+            target(x,y)=out;
+        }
+     }
+   }
+
+   template <typename Image>
+   inline void scale_image_bilinear8 (Image& target,const Image& source)
+   {
+
+      int source_width=source.width();
+      int source_height=source.height();
+
+      int target_width=target.width();
+      int target_height=target.height();
+
+      if (source_width<1 || source_height<1 ||
+          target_width<1 || target_height<1) return;
+      int x=0,y=0,xs=0,ys=0;
+      int tw2 = target_width/2;
+      int th2 = target_height/2;
+
+
+      for (y=0;y<target_height;++y)
+      {
+        ys = y*source_height/target_height;
+        int ys1 = ys+1;
+        if (ys1>=source_height)
+            ys1--;
+        unsigned yprt = y*source_height%target_height;
+        unsigned yprt1 = target_height-yprt;
+        for (x=0;x<target_width;++x)
+        {
+            xs = x*source_width/target_width;
+            if (source_width>=target_width || source_height>=target_height){
+                target(x,y)=source(xs,ys);
+                continue;
+            }
+            unsigned xprt = x*source_width%target_width;
+            unsigned xprt1 = target_width-xprt;
+            int xs1 = xs+1;
+            if (xs1>=source_width)
+                xs1--;
+
+            unsigned a = source(xs,ys);
+            unsigned b = source(xs1,ys);
+            unsigned c = source(xs,ys1);
+            unsigned d = source(xs1,ys1);
+            unsigned p,r,s;
+            // X axis
+            p = a&0xff;
+            r = b&0xff;
+            if (p!=r)
+                r = (r*xprt+p*xprt1+tw2)/target_width;
+            p = c&0xff;
+            s = d&0xff;
+            if (p!=s)
+                s = (s*xprt+p*xprt1+tw2)/target_width;
+            // Y axis
+            if (r!=s)
+                r = (s*yprt+r*yprt1+th2)/target_height;
+            target(x,y)=(0xff<<24) | (r<<16) | (r<<8) | r;
+        }
+     }
+   }
+
    inline MAPNIK_DECL void save_to_file (Image32 const& image,
                                          std::string const& file,
                                          std::string const& type) 
@@ -280,6 +425,12 @@ namespace mapnik {
    {
       save_to_file<ImageData32>(image.data(),file);
    }
+
+   inline MAPNIK_DECL std::string save_to_string(Image32 const& image,
+                                        std::string const& type)
+   {
+      return save_to_string<ImageData32>(image.data(),type);
+   }
    
 #ifdef _MSC_VER
    template MAPNIK_DECL void save_to_file<ImageData32>(ImageData32 const&,
@@ -287,6 +438,8 @@ namespace mapnik {
                                                        std::string const&);
    template MAPNIK_DECL void save_to_file<ImageData32>(ImageData32 const&,
                                                        std::string const&);
+   template MAPNIK_DECL std::string save_to_string<ImageData32>(ImageData32 const&,
+                                                       std::string const&);
    
    template MAPNIK_DECL void save_to_file<image_view<ImageData32> > (image_view<ImageData32> const&,
                                                                      std::string const&,
@@ -295,6 +448,8 @@ namespace mapnik {
    template MAPNIK_DECL void save_to_file<image_view<ImageData32> > (image_view<ImageData32> const&,
                                                                      std::string const&);
    
+   template MAPNIK_DECL std::string save_to_string<image_view<ImageData32> > (image_view<ImageData32> const&,
+                                                                     std::string const&);
 #endif
 
 }
