@@ -27,6 +27,7 @@
 #include <mapnik/config_error.hpp>
 
 // boost
+#include <boost/version.hpp>
 #include <boost/filesystem/operations.hpp>
 #include <boost/algorithm/string.hpp>
 
@@ -108,11 +109,22 @@ namespace mapnik
    }
 
    bool datasource_cache::insert(const std::string& type,const lt_dlhandle module)
-   {	      
+   {
       return plugins_.insert(make_pair(type,boost::shared_ptr<PluginInfo>
                                        (new PluginInfo(type,module)))).second;     
    }
 
+   std::vector<std::string> datasource_cache::plugin_names ()
+   {
+      std::vector<std::string> names;
+      std::map<std::string,boost::shared_ptr<PluginInfo> >::const_iterator itr;
+      for (itr = plugins_.begin();itr!=plugins_.end();++itr)
+      {
+         names.push_back(itr->first);
+      }
+      return names;
+   }
+   
    void datasource_cache::register_datasources(const std::string& str)
    {	
 #ifdef MAPNIK_THREADSAFE
@@ -121,11 +133,19 @@ namespace mapnik
 #endif
       filesystem::path path(str);
       filesystem::directory_iterator end_itr;
+ 
+
       if (exists(path) && is_directory(path))
       {
          for (filesystem::directory_iterator itr(path);itr!=end_itr;++itr )
          {
-            if (!is_directory( *itr )  && is_input_plugin(itr->leaf()))
+
+#if BOOST_VERSION < 103400 
+            if (!is_directory( *itr )  && is_input_plugin(itr->leaf()))      
+#else
+            if (!is_directory( *itr )  && is_input_plugin(itr->path().leaf()))   
+#endif
+
             {
                try 
                {
@@ -137,14 +157,14 @@ namespace mapnik
                      if (ds_name && insert(ds_name(),module))
                      {            
 #ifdef MAPNIK_DEBUG
-                        std::clog<<"registered datasource : "<<ds_name()<<std::endl;
+                        std::clog << "registered datasource : " << ds_name() << std::endl;
 #endif 
                         registered_=true;
                      }
                   }
                   else
                   {
-                     std::clog << lt_dlerror() << "\n";
+                     std::clog << "Problem loading plugin library: " << itr->string().c_str() << " (libtool error: " << lt_dlerror() << ")" << std::endl;
                   }
                }
                catch (...) {}
